from urllib2 import Request, urlopen
from subprocess import Popen, PIPE
from tempfile import NamedTemporaryFile
import json

def fetch_blueprint(name, key):	
	request = Request("https://api.apiary.io/blueprint/get/" + name)
	request.add_header('authentication', 'Token ' + key)
	request.add_header('Accept', 'application/json')
	response_body = json.loads(urlopen(request).read())
	return response_body['code']

def blueprint2json(blueprint):
	t = NamedTemporaryFile()
	p = Popen(['snowcrash', '--format', 'json', '--output', t.name], stdin=PIPE, stdout=PIPE, stderr=PIPE)
	p.communicate(blueprint)
	out = t.read()
	t.close()
	return out
	


import json
from sys import stdout
from uuid import uuid4
from time import time

def write(json_data, out=stdout, only_collection=False, pretty=False):
	result = dict()
	json_obj = json.loads(json_data)

	# Create the header
	result['version'] = 1
	result['collections'] = []
	result['globals'] =[]
	result['headerPresets'] =[]

	# Create the Environment
	result['environments'] = [ createEnvironment(json_obj) ]	

	# Create the collection
	result['collections'] = parseResourceGroups(
		json_obj['resourceGroups'], 
		result['environments'][0]['values'], 
		only_collection )

	result_out = result
	if only_collection:
		result_out = result['collections'][0]
	
	if pretty:
		json.dump(result_out, out, indent=2, separators=(',', ': '))
	else:
		json.dump(result_out, out)


def createEnvironment(json_obj):
	environment = dict()
	environment['id'] = str(uuid4())
	environment['name'] = json_obj['name']
	environment['timestamp'] = int(time())
	environment['synced'] = False
	environment['syncedFilename'] = ''
	environment['values'] = []

	for metadata in json_obj['metadata']:
		if metadata['name'] == "FORMAT":
			continue

		value = dict()
		value['name'] = metadata['name']
		value['key'] = metadata['name']
		value['value'] = metadata['value']
		value['type'] = 'text'
		environment['values'].append(value)

	return environment

def parseResourceGroups(resourceGroups, environment_vals, only_collection):
	out = []
	for resourceGroup in resourceGroups:
		collection = dict()
		collection['id'] = str(uuid4());
		collection['folders'] = []
		collection['requests'] = []
		collection['name'] = resourceGroup['name']
		collection['description'] = resourceGroup['description']
		collection['timestamp'] = int(time())
		collection['synced'] = False
		collection['remote_id'] = 0

		for resource in resourceGroup['resources']:		
			folder = dict()
			folder['id'] = str(uuid4())
			folder['name'] = resource['name']
			folder['description'] = resource['description']
			folder['order'] = []
			folder['collection_id'] = collection['id']
			folder['collection_name'] = collection['name']	
		
			sub_url = resource['uriTemplate']
			for action in resource['actions']:
				request = dict()
				request['id'] = str(uuid4())
				request['version'] = 2
				request['name'] = action['name']
				request['description'] = action['description']
				request['descriptionFormat'] = 'html'
				request['method'] = action['method']

				request['url'] = "{{HOST}}"+sub_url
				if only_collection:
					for value in environment_vals:
						if value['name'] == 'HOST':
							request['url'] = value['value'] + sub_url

				request['dataMode'] = 'params'
				request['data'] = []

				# Unsupported data				
				request['pathVariables'] = dict()
				request['tests'] = ''
				request['time'] = int(time())
				request['responses'] = []
				request['synced'] = False

				for parameter in resource['parameters']:
					request['url'] = request['url'].replace('{'+ parameter['name'] +'}', parameter['example'])

				headers = []

				# Add Authorization
				headers.append('Authorization: {{BASIC-Auth}}')

				for example in action['examples']:
					# Add Headers
					for request_ex in example['requests']:
						for header in request_ex['headers']:
							headers.append(header['name'] + ": " + header['value'])

						if len(request_ex['body']) > 0:
							request['dataMode'] = 'raw'
							request['data'] = request_ex['body']

					# Add Accept header to request based on response model (hack?)
					for response in example['responses']:
						for header in response['headers']:
							if header['name'] != 'Content-Type':
								continue
							headers.append('Accept: ' + header['value'])

				request['headers'] = '\n'.join(headers)
				# Add reference to folder to this request
				request['collectionId'] = folder['id']
				# Add reference to the request to the current folder
				folder['order'].append( request['id'] )
				# Add request json to the collection
				collection['requests'].append(request)

			# Add folder json to collection
			collection['folders'].append( folder )
		out.append(collection)
	return out



#!/usr/bin/env python
from sys import stdin, stdout, argv, exit
import argparse
import subprocess
import os
from converter import write
from apiary import blueprint2json,fetch_blueprint
def readInput():
    content = ""
    for line in stdin:
        content += line

    return content

def check_snowcrash():
    try: 
        subprocess.check_output(['which', 'snowcrash'])
    except: 
        print 'Please install snowcrash:'
        print ''
        print 'By using brew:'
        print '\tbrew install --HEAD https://raw.github.com/apiaryio/snowcrash/master/tools/homebrew/snowcrash.rb'
        print ''
        print 'By source, see:'
        print '\thttps://github.com/apiaryio/snowcrash#build'
        exit(3)

def main():

    parser = argparse.ArgumentParser(description='Generate Postman collections from Apiary API/Blueprint', formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog='Examples:\n\t' + argv[0] + ' --pretty api awattgarde\n\t' 
            + argv[0] + ' --only-collection --output awattgarde_collection.postman api awattgarde\n\t'
            + argv[0] + ' --only-collection json blueprint.json\n\t'
            +'cat blueprint.json | ' + argv[0] + ' json > awattgarde_collection.postman\n\t'  )


    subparsers = parser.add_subparsers(help='',title='subcommands', 
                                   description='valid subcommands')

    parser_blueprint = subparsers.add_parser('blueprint', description='blueprint: Read Blueprint API markup, then convert it using snowcrash, then generate Postman collection JSON.',
        help='Read Blueprint API markup, then convert it using snowcrash, then generate Postman collection JSON. Use "blueprint -h" for more help.')

    parser_json = subparsers.add_parser('json', description='json: Use prepared JSON to generate the Postman collection.',help='Use prepared JSON. Use "json -h" for more help.')
    
    parser_api = subparsers.add_parser('api', description='api: Use the Apiary API to fetch the Blueprint markup, then convert it using snowcrash, then generate Postman collection JSON.',
        help='Use the Apiary API to fetch the JSON. Use "api -h" for more help.')

    parser.add_argument('--pretty', dest='pretty', action='store_true', default=False,
                        help='generate pretty JSON')
    parser.add_argument('--only-collection', dest='only_collection', action='store_const', 
                        const=True, default=False,
                        help='generate Postman JSON for the first collection only.')
    parser_api.add_argument('key', metavar='api-key', nargs='?',
                        help='the Apiary API token. If not supplied APIARY_API_KEY environment variable is used.') 
    parser_api.add_argument('name', metavar='api-name', nargs=1,
                        help='the name of the api on apiary. I.e. testapi311 for http://docs.testapi311.apiary.io/') 
    parser_json.add_argument('input', metavar='input', type=file, nargs='?', default=stdin,
                        help='input file, formatted as JSON. If not supplied, stdin is used.') 
    parser_blueprint.add_argument('blueprint_input', metavar='input', type=file, nargs='?', default=stdin,
                        help='input file, formatted as Blueprint API Markup. If not supplied, stdin is used.') 
    parser.add_argument('--output', metavar='output', type=argparse.FileType('w'), nargs=1, default=stdout,
                        help='output file. Outputs Postman collection JSON. If not supplied, stdout is used.')

    args = parser.parse_args()

    input = ''
    if hasattr(args, 'input'):
        # JSON mode
        input = args.input.read()
    elif hasattr(args, 'blueprint_input'):
        # blueprint mode
        input = blueprint2json(args.blueprint_input.read())
    else:
        # API mode
        check_snowcrash()

        apikey = None
        if args.key != None:
            apikey = args.key[0]
        else:
            apikey = os.environ.get('APIARY_API_KEY')

        if apikey == None:
            print 'Please provide an api-key or set APIARY_API_KEY'
            exit(4)

        blueprint = fetch_blueprint(args.name[0], apikey)
        input = blueprint2json(blueprint)
        
    output = args.output    
    if args.output != stdout:
        output = output[0]

    write(input, output, args.only_collection, args.pretty)

if  __name__ =='__main__':
    main()




from distutils.core import setup
setup(
  name = 'apiary2postman',
  packages = ['apiary2postman'], # this must be the same as the name above
  version = '0.4.1',
  description = 'A tool for converting Blueman API markup from Apiary.io to Postman collection/dumps',
  author = 'Erik Jonsson Thoren',
  author_email = 'thecopy@gmail.com',
  url = 'https://github.com/thecopy/apiary2postman', # use the URL to the github repo
  download_url = 'https://github.com/thecopy/apiary2postman/tarball/0.4.1', # I'll explain this in a second
  keywords = ['apiary', 'blueman', 'postman'], # arbitrary keywords
  classifiers = [],
  entry_points={
        'console_scripts': [
            'apiary2postman = apiary2postman.apiary2postman:main',
        ]
    }
)